#ifndef SAW_HPP
#define SAW_HPP

class step2d
{
public:
	int x,y;
	char d; // n,s,e,w
private:
	void move(char d)
	{
		if (d == 'n') ++y;
		else if (d == 's') --y;
		else if (d == 'e') ++x;
		else if (d == 'w') --x;
		else assert(false);
	}
public:
	step2d(char d_) : x(0), y(0), d(d_) { move(d); };
	step2d(const step2d &last_step, char d_) : x(last_step.x), y(last_step.y), d(d_) { move(d); }
	bool is_at_origin(void) const { return x == 0 and y == 0; }
	friend bool operator==(const step2d &s1, const step2d &s2) { return s1.x == s2.x and s1.y == s2.y; }
};

class step3d
{
public:
	int x,y,z;
	char d; // n,s,e,w,u,d
private:
	void move(char d)
	{
		if (d == 'n') ++y;
		else if (d == 's') --y;
		else if (d == 'e') ++x;
		else if (d == 'w') --x;
		else if (d == 'u') ++z;
		else if (d == 'd') --z;
		else assert(false);
	}
public:
	step3d(char d_) : x(0), y(0), z(0), d(d_) { move(d); };
	step3d(const step3d &last_step, char d_) : x(last_step.x), y(last_step.y), z(last_step.z), d(d_) { move(d); }
	bool is_at_origin(void) const { return x == 0 and y == 0 and z == 0; }
	friend bool operator==(const step3d &s1, const step3d &s2) { return s1.x == s2.x and s1.y == s2.y and s1.z == s2.z; }
};

template <typename step>
class saw
{
private:
	vector<step> path;
public:
	saw(char d) : path() { path.push_back(step(d)); }
	size_t length(void) const { return path.size(); }
	char last_direction(void) const { return (length() > 0 ? path[length()-1].d : ' '); }
	const step& last_step(void) const { assert(length() > 0) ; return path[length()-1]; }
	const step& operator[](size_t i) const { assert(i < length()); return path[i]; }
	bool is_allowed(char d)
	{
		step next_step(last_step(),d);
		if (next_step.is_at_origin())
			return false;
		for (size_t i = 0; i < length(); ++i)
			if (path[i] == next_step)
				return false;
		return true;
	}
	friend saw operator+(const saw &walk, char d)
	{
		saw walk2(walk);
		walk2.path.push_back(step(walk2.last_step(),d));
		return walk2;
	}
	friend ostream& operator<<(ostream &os, const saw &walk)
	{
		for (size_t i = 0; i < walk.length(); ++i)
			os << walk[i].d;
		return os;
	}
};

#endif // SAW_HPP