#include <cassert>

#include <utility>
using std::pair;

#include <vector>
using std::vector;

#include <queue>
using std::queue;

#include <iostream>
using std::cerr;
using std::cout;
using std::endl;
using std::ostream;

#include <cmath>
using std::exp;

#include <algorithm>
using std::max;
using std::min;

#include "saw.hpp"

const size_t N = 14;
typedef saw<step2d> saw2d;
queue<saw2d> Q;

typedef pair<double,size_t> weighted_observation;
vector<weighted_observation> r2[N];

size_t energy(const saw2d &walk)
{
	size_t interactions = 0;
	for (size_t i = 2; i < walk.length(); ++i)
	{
		const int xx = abs(walk[i].x);
		const int yy = abs(walk[i].y);
		if ((xx == 0 and yy == 1) or (xx == 1 and yy == 0))
			++interactions;
		for (size_t j = 0; j < i-1; ++j)
		{
			assert(i < walk.length());
			assert(j < walk.length());
			const int xx = abs(walk[i].x - walk[j].x);
			const int yy = abs(walk[i].y - walk[j].y);
			if ((xx == 0 and yy == 1) or (xx == 1 and yy == 0))
				++interactions;
		}
	}
	return interactions;
}

double dist_sqr(const saw2d &walk)
{
	const double x = walk.last_step().x;
	const double y = walk.last_step().y;
	return x*x+y*y;
}

int main()
{
	Q.push(saw2d('n'));
	r2[0].push_back(weighted_observation(1.0,0));
	size_t n = Q.front().length();
	while (!Q.empty())
	{
		saw2d walk = Q.front();
		Q.pop();
		const size_t l = walk.length();
		if (l < N)
		{
			const char dir[] = "nswe";
			const char opp[] = "snew";
			for (size_t i = 0; i < 4; ++i)
				if (walk.last_direction() != opp[i] and walk.is_allowed(dir[i]))
				{
					Q.push(walk + dir[i]);
					const size_t nn = Q.front().length();
					if (nn > n)
					{
						cerr << "Length " << n << " enumerated" << endl;
						n = nn;
					}
					const saw2d &last_walk = Q.back();
					r2[l].push_back(weighted_observation( dist_sqr(last_walk), energy(last_walk) ));
				}
		}
	}
		
	for (size_t l = 0; l < N; ++l)
	{
		for (double x = -10.0; x < 10.0; x += 0.2)
		{
			double sum_r2_w = 0.0;
			double sum_w = 0.0;
			for (size_t k = 0; k < r2[l].size(); ++k)
			{
				const double w = exp(x*r2[l][k].second);
				sum_r2_w += r2[l][k].first*w;
				sum_w += w;
			}
			cout << (l+1) << "\t" << x << "\t" << sum_r2_w/sum_w << endl;
		}
		cout << endl << endl;
	}
	return 0;
}

